@@ -6,6 +6,7 @@
 # node ::= SCALAR | sequence | mapping
 # sequence ::= SEQUENCE-START node* SEQUENCE-END
 # mapping ::= MAPPING-START (node node)* MAPPING-END
+
 # To Do:
 # - Make encode stuff work
 
@@ -15,7 +16,7 @@ use warnings;
 
 use YAML::Perl::Error;
 use YAML::Perl::Events;
-use YAML::Perl::Writer;
+use YAML::Perl::Stream;
 
 package YAML::Perl::Error::Emitter;
 use YAML::Perl::Error -base;
@@ -35,15 +36,14 @@ field 'allow_block';
 package YAML::Perl::Emitter;
 use YAML::Perl::Processor -base;
 
-field next_layer => 'writer';
-field 'writer_class', -init => '"YAML::Perl::Writer"';
-field 'writer', -init => '$self->create("writer")';
+field next_layer => '';
 
 use constant DEFAULT_TAG_PREFIXES => {
     '!' => '!',
     'tag:yaml.org,2002:' => '!!',
 };
 
+field 'stream';
 field 'encoding';
 field 'states' => [];
 field 'state' => 'expect_stream_start'; # Made this a function name instead of pointer
@@ -84,12 +84,24 @@ sub init {
         $p{best_line_break} = delete $p{line_break};
     }
     $self->SUPER::init(%p);
+    if (not $self->stream) {
+        my $output = '';
+        $self->stream(YAML::Perl::Stream->open(\ $output)); 
+    }
 }
 
 sub emit {
     my $self = shift;
     my $events = $self->events;
-    push @$events, @_;
+    if (ref($_[0]) eq 'CODE') {
+        my $iterator = shift;
+        while (my $event = $iterator->()) {
+            push @$events, $event; 
+        }
+    }
+    else {
+        push @$events, @_;
+    }
 
     while (not $self->need_more_events()) {
         $self->event(shift @$events);
@@ -97,7 +109,7 @@ sub emit {
         $self->$state();
         $self->event(undef);
     }
-    return ${$self->writer->stream->buffer};
+    return ${$self->stream->buffer};
 }
 
 sub need_more_events {
@@ -147,8 +159,8 @@ sub need_events {
 
 sub increase_indent {
     my $self = shift;
-    my $flow = @_ ? shift : False;
-    my $indentless = @_ ? shift : False;
+    my $flow = shift || False;
+    my $indentless = shift || False;
     push @{$self->indents}, $self->indent;
     if (not defined $self->indent) {
         if ($flow) {
@@ -194,7 +206,7 @@ sub expect_first_document_start {
 
 sub expect_document_start {
     my $self = shift;
-    my $first = @_ ? shift : False;
+    my $first = shift || False;
     if ($self->event->isa('YAML::Perl::Event::DocumentStart')) {
         if ($self->event->version) {
             my $version_text = $self->prepare_version($self->event->version);
@@ -218,7 +230,6 @@ sub expect_document_start {
             not $self->event->tags and
             not $self->check_empty_document()
         );
-        $implicit = False; # XXX Different than PyYaml
         if (not $implicit) {
             $self->write_indent();
             $self->write_indicator('---', True);
@@ -315,11 +326,7 @@ sub expect_node {
 
 sub expect_alias {
     my $self = shift;
-    if (not $self->event->anchor) {
-        throw YAML::Perl::Error::Emitter("anchor is not specified for alias");
-    }
-    $self->process_anchor("*");
-    $self->state(pop @{$self->states});
+    die 'expect_alias';
 }
 
 sub expect_scalar {
@@ -332,157 +339,47 @@ sub expect_scalar {
 
 sub expect_flow_sequence {
     my $self = shift;
-
-    $self->write_indicator('[', True, whitespace => True);
-    $self->flow_level($self->flow_level + 1);
-    $self->increase_indent(True);
-    $self->state('expect_first_flow_sequence_item');
+    die 'expect_flow_sequence';
 }
 
 sub expect_first_flow_sequence_item {
-    my $self = shift;
-
-    if ($self->event->isa('YAML::Perl::Event::SequenceEnd')) {
-        $self->indent(pop @{$self->indents});
-        $self->flow_level -= 1;
-        $self->write_indicator(']', False);
-        $self->state(pop @{$self->states});
-    }
-    else {
-        if ($self->canonical or $self->column > $self->best_width) {
-            $self->write_indent();
-        }
-        push @{$self->states}, 'expect_flow_sequence_item';
-        $self->expect_node(sequence => True);
-    }
+    die 'expect_first_flow_sequence_item';
 }
 
 sub expect_flow_sequence_item {
-    my $self = shift;
-
-    if ($self->event->isa('YAML::Perl::Event::SequenceEnd')) {
-        $self->indent(pop @{$self->indents});
-        $self->flow_level($self->flow_level - 1);
-        if ($self->canonical) {
-            $self->write_indicator(',', False);
-            $self->write_indent();
-        }
-        $self->write_indicator(']', False);
-        $self->state(pop @{$self->states});
-    }
-    else {
-        $self->write_indicator(',', False);
-        if ($self->canonical or $self->column > $self->best_width) {
-            $self->write_indent();
-        }
-        push @{$self->states}, 'expect_flow_sequence_item';
-        $self->expect_node(sequence => True);
-    }
+    die 'expect_flow_sequence_item';
 }
 
 sub expect_flow_mapping {
-    my $self = shift;
-    $self->write_indicator('{', True, whitespace => True);
-    $self->flow_level($self->flow_level + 1);
-    $self->increase_indent(True);
-    $self->state('expect_first_flow_mapping_key');
+    die 'expect_flow_mapping';
 }
 
 sub expect_first_flow_mapping_key {
-    my $self = shift;
-    if ($self->event->isa('YAML::Perl::Event::MappingEnd')) {
-        $self->indent(pop @{$self->indents});
-        $self->flow_level($self->flow_level - 1);
-        $self->write_indicator('}', False);
-        self->state(pop @{$self->states});
-    }
-    else {
-        if ($self->canonical or $self->column > $self->best_width) {
-            $self->write_indent();
-        }
-        if (not $self->canonical and $self->check_simple_key()) {
-            push @{$self->states}, 'expect_flow_mapping_simple_value';
-            $self->expect_node(mapping => True, simple_key => True);
-        }
-        else {
-            $self->write_indicator('?', True);
-            push @{$self->states}, 'expect_flow_mapping_value';
-            $self->expect_node(mapping => True);
-        }
-    }
+    die 'expect_first_flow_mapping_key';
 }
 
 sub expect_flow_mapping_key {
-    my $self = shift;
-    if ($self->event->isa('YAML::Perl::Event::MappingEnd')) {
-        $self->indent(pop, @{$self->indents});
-        $self->flow_level($self->flow_level - 1);
-        if ($self->canonical) {
-            $self->write_indicator(',', False);
-            $self->write_indent();
-        }
-        $self->write_indicator('}', False);
-        $self->state(pop @{$self->states});
-    }
-    else {
-        $self->write_indicator(',', False);
-        if ($self->canonical or $self->column > $self->best_width) {
-            $self->write_indent();
-        }
-        if (not $self->canonical and $self->check_simple_key()) {
-            push @{$self->states}, 'expect_flow_mapping_simple_value';
-            $self->expect_node(mapping => True, simple_key => True);
-        }
-        else {
-            $self->write_indicator('?', True);
-            push @{$self->states}, 'expect_flow_mapping_value';
-            $self->expect_node(mapping => True);
-        }
-    }
+    die 'expect_flow_mapping_key';
 }
 
 sub expect_flow_mapping_simple_value {
-    my $self = shift;
-    $self->write_indicator(':', False);
-    push @{$self->states}, 'expect_flow_mapping_key';
-    $self->expect_node(mapping => True);
+    die 'expect_flow_mapping_simple_value';
 }
 
 sub expect_flow_mapping_value {
-    my $self = shift;
-    if ($self->canonical or $self->column > $self->best_width) {
-        $self->write_indent();
-    }
-    $self->write_indicator(':', True);
-    push @{$self->states}, 'expect_flow_mapping_key';
-    $self->expect_node(mapping => True);
+    die 'expect_flow_mapping_value';
 }
 
 sub expect_block_sequence {
-    my $self = shift;
-    my $indentless = ($self->mapping_context and not $self->indention);
-    $self->increase_indent(False, $indentless);
-    $self->state('expect_first_block_sequence_item');
+    die 'expect_block_sequence';
 }
 
 sub expect_first_block_sequence_item {
-    my $self = shift;
-    return $self->expect_block_sequence_item(False);
+    die 'expect_first_block_sequence_item';
 }
 
 sub expect_block_sequence_item {
-    my $self = shift;
-    my $first = @_ ? shift : False;
-    if (not $first and $self->event->isa('YAML::Perl::Event::SequenceEnd')) {
-        $self->indent(pop @{$self->indents});
-        $self->state(pop @{$self->states});
-    }
-    else {
-        $self->write_indent();
-        $self->write_indicator('-', True, indention => True);
-        push @{$self->states}, 'expect_block_sequence_item';
-        $self->expect_node(sequence => True);
-    }
+    die 'expect_block_sequence_item';
 }
 
 sub expect_block_mapping {
@@ -498,7 +395,7 @@ sub expect_first_block_mapping_key {
 
 sub expect_block_mapping_key {
     my $self = shift;
-    my $first = @_ ? shift : False;
+    my $first = shift || False;
     if (not $first and $self->event->isa('YAML::Perl::Event::MappingEnd')) {
         $self->indent(pop @{$self->indents});
         $self->state(pop @{$self->states});
@@ -529,12 +426,7 @@ sub expect_block_mapping_value {
 }
 
 sub check_empty_sequence {
-    my $self = shift;
-    return (
-        $self->event->isa('YAML::Perl::Event::SequenceStart') and
-        @{$self->events} and
-        $self->events->[0]->isa('YAML::Perl::Event::MappingEnd')
-    );
+    die 'check_empty_sequence';
 }
 
 sub check_empty_mapping {
@@ -690,7 +582,7 @@ sub choose_scalar_style {
             return $self->event->style
         }
     }
-    if (not $self->event->style or $self->event->style eq '\'') {
+    if (not $self->event->style or $self->event->style == '\'') {
         if (
             $self->analysis->allow_single_quoted and
             not ($self->simple_key_context and $self->analysis->multiline)
@@ -733,13 +625,7 @@ sub process_scalar {
 }
 
 sub prepare_version {
-    my $self = shift;
-    my $version = shift;
-    my ($major, $minor) = split('\.', $version);
-    if ($major != 1) {
-        throw YAML::Perl::Error::Emitter->("unsupported YAML version: $major.$minor");
-    }
-    return "$major.$minor";
+    die 'prepare_version';
 }
 
 sub prepare_tag_handle {
@@ -867,17 +753,7 @@ sub prepare_tag {
 }
 
 sub prepare_anchor {
-    my $self = shift;
-    my $anchor = shift;
-    if (not $anchor) {
-        throw YAML::Perl::Error::Emitter("anchor must not be empty");
-    }
-    foreach my $ch (split('', $anchor)) {
-        if ($ch !~ /[0-9A-Za-z-_]/) {
-            throw YAML::Perl::Error::Emitter("invalid character $ch in the anchor $anchor");
-        }
-    }
-    return $anchor;
+    die 'prepare_anchor';
 }
 
 sub analyze_scalar {
@@ -925,7 +801,7 @@ sub analyze_scalar {
 
     # Last character or followed by a whitespace.
     my $followed_by_space =
-        (length($scalar) == 1 or $scalar =~ /^.[\0 \t\r\n\x85\x{2028}\x{2029}]/s);
+        (length($scalar) == 1 or $scalar =~ /.[\0 \t\r\n\x85\x{2028}\x{2029}]/);
 
     # The current series of whitespaces contain plain spaces.
     my $spaces = False;
@@ -1020,13 +896,13 @@ sub analyze_scalar {
                 }
             }
             elsif ($breaks) {
-                if ($ch eq ' ') {      # break+ space+
+                if ($ch == ' ') {      # break+ space+
                     $spaces = True;
                 }
             }
             else {
                 $leading = ($index == 0);
-                if ($ch eq ' ') {      # space+
+                if ($ch == ' ') {      # space+
                     $spaces = True;
                 }
                 else {                 # break+
@@ -1066,7 +942,7 @@ sub analyze_scalar {
         }
 
         # Series of whitespaces reach the end.
-        if (($spaces or $breaks) and ($index == (length($scalar) - 1))) {
+        if (($spaces or $breaks) and ($index == length($scalar)-1)) {
             if ($spaces and $breaks) {
                 $mixed_breaks_spaces = True;
             }
@@ -1090,7 +966,7 @@ sub analyze_scalar {
         $preceeded_by_space = ($ch =~ /^[\0 \t\r\n\x85\x{2028}\x{2029}]$/);
         $followed_by_space = (
             $index + 1 >= length($scalar) or
-            substr($scalar, index + 1, 1) =~ /^[\0\ \t\r\n\x85\x{2028}\x{2029}]$/
+            substr($scalar, index+1, 1) =~ /^[\0\ \t\r\n\x85\x{2028}\x{2029}]$/
         );
     }
 
@@ -1155,15 +1031,15 @@ sub analyze_scalar {
 
 sub flush_stream {
     my $self = shift;
-    if ($self->writer->stream->can('flush')) {
-        $self->writer->stream->flush();
+    if ($self->stream->can('flush')) {
+        $self->stream->flush();
     }
 }
 
 sub write_stream_start {
     my $self = shift;
     if ($self->encoding and $self->encoding =~ /^utf-16/) {
-        $self->writer->write("\xff\xfe");
+        $self->stream->write("\xff\xfe");
     }
 }
 
@@ -1193,7 +1069,7 @@ sub write_indicator {
     if ($self->encoding) {
 #         my $data = $data->encode($self->encoding);
     }
-    $self->writer->write($data);
+    $self->stream->write($data);
 }
 
 sub write_indent {
@@ -1212,7 +1088,7 @@ sub write_indent {
         if ($self->encoding) {
             # $data = $data->encode($self->encoding); #XXX
         }
-        $self->writer->write($data);
+        $self->stream->write($data);
     }
 }
 
@@ -1229,18 +1105,11 @@ sub write_line_break {
     if ($self->encoding) {
 #         $data = $data->encode($self->encoding);
     }
-    $self->writer->write($data);
+    $self->stream->write($data);
 }
 
 sub write_version_directive {
-    my $self = shift;
-    my $version_text = shift;
-    my $data = "%YAML $version_text";
-    if ($self->encoding) {
-#         $data = $data->encode($self->encoding);
-    }
-    $self->writer->write($data);
-    $self->write_line_break();
+    die 'write_version_directive';
 }
 
 sub write_tag_directive {
@@ -1278,7 +1147,7 @@ sub write_single_quoted {
                     if ($self->encoding) {
                         # my $data = $data->encode($self->encoding);
                     }
-                    $self->writer->write($data);
+                    $self->stream->write($data);
                 }
                 $start = $end;
             }
@@ -1311,18 +1180,18 @@ sub write_single_quoted {
                     if ($self->encoding) {
                         # $data = $data->encode($self->encoding);
                     }
-                    $self->writer->write($data);
+                    $self->stream->write($data);
                     $start = $end;
                 }
             }
         }
-        if ($ch and $ch eq '\'') {
+        if ($ch eq '\'') {
             my $data = '\'\'';
             $self->column($self->column + 2);
             if ($self->encoding) {
                 # $data = $data->encode($self->encoding);
             }
-            $self->writer->write($data);
+            $self->stream->write($data);
             $start = $end + 1;
         }
         if (defined $ch) {
@@ -1334,108 +1203,8 @@ sub write_single_quoted {
     $self->write_indicator('\'', False);
 }
 
-use constant ESCAPE_REPLACEMENTS => {
-    "\0" => '0',
-    "\x07" => 'a',
-    "\x08" => 'b',
-    "\x09" => 't',
-    "\x0A" => 'n',
-    "\x0B" => 'v',
-    "\x0C" => 'f',
-    "\x0D" => 'r',
-    "\x1B" => 'e',
-    "\"" => '"',
-    "\\" => '\\',
-    "\x85" => 'N',
-    "\xA0" => '_',
-    "\x{2028}" => 'L',
-    "\x{2029}" => 'P',
-};
-
 sub write_double_quoted {
-    my $self = shift;
-    my $text = shift;
-    my $split = @_ ? shift : True;
-
-    $self->write_indicator('"', True);
-    my $start = 0;
-    my $end = 0;
-    while ($end <= length($text)) {
-        my $ch = undef;
-        if ($end < length($text)) {
-            $ch = substr($text, $end, 1);
-        }
-        if (not defined $ch or
-            $ch =~ /^[\"\\\x85\x{2028}\x{2029}\x{FEFF}]$/ or
-            not (
-                $ch ge "\x20" and $ch le "\x7E" or
-                ($self->allow_unicode and
-                    (
-                        $ch ge "\xA0" and $ch le "\x{D7FF}" or
-                        $ch ge "\x{E000}" and $ch le "\x{FFFD}"
-                    )
-                )
-            )) {
-            if ($start < $end) {
-                my $data = substr($text, $start, $end - $start);
-                $self->column($self->column + length($data));
-                if ($self->encoding) {
-                    # $data = $data; # .encode(self.encoding)
-                }
-                $self->writer->write($data);
-                $start = $end;
-            }
-            if (defined $ch) {
-                my $data;
-                if (defined ESCAPE_REPLACEMENTS->{$ch}) {
-                    $data = '\\' . ESCAPE_REPLACEMENTS->{$ch};
-                }
-                elsif ($ch le "\xFF") {
-                    $data = sprintf "\\x%02X", ord($ch);
-                }
-#                 elsif ($ch le "\x{ffff}") {
-#                     $data = sprintf "\\u%04X", ord($ch);
-#                 }
-                else {
-                    $data = printf "\\U%08X", ord($ch);
-                }
-                $self->column($self->column + length($data));
-                if ($self->encoding) {
-                    # $data = $data; # .encode(self.encoding)
-                }
-                $self->writer->write($data);
-                $start = $end + 1;
-            }
-        }
-        if ($end > 0 and $end < (length($text) - 1) and
-            ($ch eq ' ' or $start >= $end) and
-            ($self->column + ($end - $start)) > $self->best_width and
-            $split
-        ) {
-            my $data = substr($text, $start, $end - $start) . '\\';
-            if ($start < $end) {
-                $start = $end;
-            }
-            $self->column($self->column + length($data));
-            if ($self->encoding) {
-                # $data = $data; # .encode(self.encoding)
-            }
-            $self->writer->write($data);
-            $self->write_indent();
-            $self->whitespace(False);
-            $self->indention(False);
-            if (substr($text, $start, 1) eq ' ') {
-                $data = '\\';
-                $self->column($self->column + length($data));
-                if ($self->encoding) {
-                    # $data = $data; # .encode(self.encoding)
-                }
-                $self->writer->write($data);
-            }
-        }
-        $end += 1;
-    }
-    $self->write_indicator('"', False);
+    die 'write_double_quoted';
 }
 
 sub determine_chomp {
@@ -1463,7 +1232,7 @@ sub write_plain {
         if ($self->encoding) {
 #             $data = $data->encode($self->encoding);
         }
-        $self->writer->write($data);
+        $self->stream->write($data);
     }
     $self->whitespace(False);
     $self->indention(False);
@@ -1491,7 +1260,7 @@ sub write_plain {
                     if ($self->encoding) {
 #                         $data = $data->encode($self->encoding)
                     }
-                    $self->writer->write($data);
+                    $self->stream->write($data);
                 }
                 $start = $end;
             }
@@ -1522,7 +1291,7 @@ sub write_plain {
                 if ($self->encoding) {
 #                     $data = $data->encode($self->encoding);
                 }
-                $self->writer->write($data);
+                $self->stream->write($data);
                 $start = $end;
             }
         }
