@@ -102,11 +102,14 @@ sub parse {
         return @events;
     }
     else {
-        return $self->check_event() ? $self->get_event() : ();
+        return sub {
+            return $self->check_event() ? $self->get_event() : undef;
+        };
     }
 }
 
 sub check_event {
+    # print "+check_event\n";
     my $self = shift;
     my @choices = @_;
     if (not defined $self->current_event) {
@@ -129,6 +132,7 @@ sub check_event {
 }
 
 sub peek_event {
+    # print "+peek_event\n";
     my $self = shift;
     if (not defined $self->current_event) {
         if (my $state = $self->state) {
@@ -139,7 +143,11 @@ sub peek_event {
 }
 
 sub get_event {
+    # print "+get_event\n";
     my $self = shift;
+    # print $self->state . "\n";
+    # print "  " . $self->scanner->peek_token() . "\n";
+    # print "  [@{$self->scanner->tokens}]\n";
     if (not defined $self->current_event) {
         if (my $state = $self->state) {
             $self->current_event($self->$state());
@@ -147,6 +155,7 @@ sub get_event {
     }
     my $value = $self->current_event;
     $self->current_event(undef);
+    # print "    $value\n";
     return $value;
 }
 
@@ -268,39 +277,11 @@ sub parse_document_content {
 
 sub process_directives {
     my $self = shift;
-    $self->yaml_version(undef);
-    $self->tag_handles({});
+    $self->{yaml_version} = undef;
+    $self->{tag_handles} = {};
     while ($self->scanner->check_token('YAML::Perl::Token::Directive')) {
-        my $token = $self->scanner->get_token();
-        if ($token->name eq 'YAML') {
-            if (defined($self->yaml_version)) {
-                throw YAML::Perl::Error::Parser(
-                    "found duplicate YAML directive", $token->start_mark);
-            }
-            my ($major, $minor) = split('\.', $token->value);
-            if ($major != 1) {
-                throw YAML::Perl::Error::Parser(
-                    "found incompatible YAML document (version 1.* is required)",
-                    $token->start_mark);
-            }
-            $self->yaml_version($token->value);
-        }
-        elsif ($token->name eq 'TAG') {
-            # XXX tags not done yet
-        }
+        XXX my $token = $self->get_token();
     }
-    my $value;
-    if (keys(%{$self->tag_handles})) {
-        # XXX tags not done yet
-        #value = self.yaml_version, self.tag_handles.copy()
-    }
-    else {
-        $value = $self->yaml_version;
-    }
-#     for key in self.DEFAULT_TAGS:
-#         if key not in self.tag_handles:
-#             self.tag_handles[key] = self.DEFAULT_TAGS[key]
-    return $value;
 }
 
 sub parse_block_node {
@@ -324,7 +305,7 @@ sub parse_node {
     
     my $event;
     if ($self->scanner->check_token('YAML::Perl::Token::Alias')) {
-        my $token = $self->scanner->get_token();
+        my $token = $self->get_token();
         $event = YAML::Perl::Event::Alias->new(
             anchor     => $token->value,
             start_mark => $token->start_mark,
@@ -397,7 +378,7 @@ sub parse_node {
             if ($self->scanner->check_token('YAML::Perl::Token::Scalar')) {
                 my $token = $self->scanner->get_token();
                 $end_mark = $token->end_mark;
-                if (($token->plain and not defined $tag) or ($tag || '') eq '!') {
+                if (($token->plain and not defined $tag) or $tag eq '!') {
                     $implicit = [True, False];
                 }
                 elsif (not defined $tag) {
@@ -439,7 +420,7 @@ sub parse_node {
                     end_mark => $end_mark,
                     flow_style => True,
                 );
-                $self->state('parse_flow_mapping_first_key');
+                $self->state('parse_flow_mapping_first_entry');
             }
             elsif ($self->scanner->check_token('YAML::Perl::Token::BlockSequenceStart')) {
                 $end_mark = $self->scanner->peek_token()->end_mark;
@@ -497,35 +478,7 @@ sub parse_block_sequence_first_entry {
 
 sub parse_block_sequence_entry {
     my $self = shift;
-    if ($self->scanner->check_token('YAML::Perl::Token::BlockEntry')) {
-        my $token = $self->scanner->get_token();
-        if (not $self->scanner->check_token(qw(
-            YAML::Perl::Token::BlockEntry
-            YAML::Perl::Token::BlockEnd
-        ))) {
-            push @{$self->states}, 'parse_block_sequence_entry';
-            return $self->parse_block_node();
-        }
-        else {
-            $self->state('parse_block_sequence_entry');
-            return $self->process_empty_scalar($token->end_mark);
-        }
-    }
-    if (not $self->scanner->check_token('YAML::Perl::Token::BlockEnd')) {
-        my $token = $self->scanner->peek_token();
-        throw YAML::Perl::Error::Parser(
-            "while parsing a block collection", $self->marks->[-1],
-            "expected <block end>, but found ", $token->id, $token->start_mark
-        );
-    }
-    my $token = $self->scanner->get_token();
-    my $event = YAML::Perl::Event::SequenceEnd->new(
-        start_mark => $token->start_mark,
-        end_mark => $token->end_mark,
-    );
-    $self->state(pop @{$self->states});
-    pop @{$self->marks};
-    return $event;
+    die "parse_block_sequence_entry";
 }
 
 sub parse_indentless_sequence_entry {
@@ -561,7 +514,7 @@ sub parse_block_mapping_key {
         my $token = $self->scanner->peek_token();
         throw YAML::Perl::Error::Parser(
             "while parsing a block mapping", $self->marks->[-1],
-            "expected <block end>, but found ", $token->id, $token->start_mark
+            "expected <block end>, but found %r", $token->id, $token->start_mark
         );
     }
     my $token = $self->scanner->get_token();
@@ -600,57 +553,12 @@ sub parse_block_mapping_value {
 
 sub parse_flow_sequence_first_entry {
     my $self = shift;
-    my $token = $self->scanner->get_token();
-    push @{$self->marks}, $token->start_mark;
-    return $self->parse_flow_sequence_entry(True);
+    die "parse_flow_sequence_first_entry";
 }
 
 sub parse_flow_sequence_entry {
     my $self = shift;
-    my $first = @_ ? shift : False;
-    if (not $self->scanner->check_token('YAML::Perl::Token::FlowSequenceEnd')) {
-        if (not $first) {
-            if ($self->scanner->check_token('YAML::Perl::Token::FlowEntry')) {
-                $self->scanner->get_token();
-            }
-            else {
-                my $token = $self->scanner->peek_token();
-                throw YAML::Perl::Error::Parser(
-                    "while parsing a flow sequence",
-                    $self->marks->[-1],
-                    "expected ',' or ']', but got %r",
-                    $token->id,
-                    $token->start_mark
-                );
-            }
-        }
-        
-        if ($self->scanner->check_token('YAML::Perl::Token::Key')) {
-            my $token = $self->scanner->peek_token();
-            my $event = YAML::Perl::Event::MappingStart->new(
-                anchor => undef,
-                tag => undef,
-                implicit => True,
-                start_mark => $token->start_mark,
-                end_mark => $token->end_mark,
-                flow_style => True,
-            );
-            $self->state('parse_flow_sequence_entry_mapping_key');
-            return $event;
-        }
-        elsif (not $self->scanner->check_token('YAML::Perl::Token::FlowSequenceEnd')) {
-            push @{$self->states}, 'parse_flow_sequence_entry';
-            return $self->parse_flow_node();
-        }
-    }
-    my $token = $self->scanner->get_token();
-    my $event = YAML::Perl::Event::SequenceEnd->new(
-        start_mark => $token->start_mark,
-        end_mark => $token->end_mark,
-    );
-    $self->state(pop @{$self->states});
-    pop @{$self->marks};
-    return $event;
+    die "parse_flow_sequence_entry";
 }
 
 sub parse_flow_sequence_entry_mapping_key {
@@ -670,89 +578,22 @@ sub parse_flow_sequence_entry_mapping_end {
 
 sub parse_flow_mapping_first_key {
     my $self = shift;
-    my $token = $self->scanner->get_token();
-    push @{$self->marks}, $token->start_mark;
-    return $self->parse_flow_mapping_key(True)
+    die "parse_flow_mapping_first_key";
 }
 
 sub parse_flow_mapping_key {
     my $self = shift;
-    my $first = @_ ? shift : False;
-
-    if (not $self->scanner->check_token('YAML::Perl::Token::FlowMappingEnd')) {
-        if (not $first) {
-            if ($self->scanner->check_token('YAML::Perl::Token::FlowEntry')) {
-                $self->scanner->get_token();
-            }
-            else {
-                my $token = $self->scanner->peek_token();
-                throw YAML::Perl::Error::Parser(
-                    "while parsing a flow mapping",
-                    $self->marks->[-1],
-                    "expected ',' or '}', but got %r",
-                    $token->id,
-                    $token->start_mark
-                );
-            }
-        }
-        if ($self->scanner->check_token('YAML::Perl::Token::Key')) {
-            my $token = $self->scanner->get_token();
-            if (not $self->scanner->check_token(
-                'YAML::Perl::Token::Value',
-                'YAML::Perl::Token::FlowEntry',
-                'YAML::Perl::Token::FlowMappingEnd',
-            )) {
-                push @{$self->states}, 'parse_flow_mapping_value';
-                return $self->parse_flow_node();
-            }
-            else {
-                $self->state('parse_flow_mapping_value');
-                return $self->process_empty_scalar($token->end_mark);
-            }
-        }
-        elsif (not $self->scanner->check_token('YAML::Perl::Token::FlowMappingEnd')) {
-            push @{$self->states}, 'parse_flow_mapping_empty_value';
-            return $self->parse_flow_node();
-        }
-    }
-    my $token = $self->scanner->get_token();
-    my $event = YAML::Perl::Event::MappingEnd->new(
-        start_mark => $token->start_mark,
-        end_mark => $token->end_mark,
-    );
-    $self->state(pop @{$self->states});
-    pop @{$self->marks};
-    return $event;
+    die "parse_flow_mapping_key";
 }
 
 sub parse_flow_mapping_value {
     my $self = shift;
-
-    if ($self->scanner->check_token('YAML::Perl::Token::Value')) {
-        my $token = $self->scanner->get_token();
-        if (not $self->scanner->check_token(
-            'YAML::Perl::Token::FlowEntry',
-            'YAML::Perl::Token::FlowMappingEnd',
-        )) {
-            push @{$self->states}, 'parse_flow_mapping_key';
-            return $self->parse_flow_node();
-        }
-        else {
-            $self->state('parse_flow_mapping_key');
-            return $self->process_empty_scalar($token->end_mark);
-        }
-    }
-    else {
-        $self->state('parse_flow_mapping_key');
-        my $token = $self->scanner->peek_token();
-        return $self->process_empty_scalar($token->start_mark);
-    }
+    die "parse_flow_mapping_value";
 }
 
 sub parse_flow_mapping_empty_value {
     my $self = shift;
-    $self->state('parse_flow_mapping_key');
-    return $self->process_empty_scalar($self->scanner->peek_token()->start_mark);
+    die "parse_flow_mapping_empty_value";
 }
 
 sub process_empty_scalar {
@@ -760,7 +601,7 @@ sub process_empty_scalar {
     return YAML::Perl::Event::Scalar->new(
         anchor     => undef,
         tag        => undef,
-        implicit   => [True, False],
+        implicit   => 1,     # what does (True, False) mean??
         value      => '',
         start_mark => $mark,
         end_mark   => $mark
